/**	
 * Backbone-relational.js 0.3
 * (c) 2011 Paul Uithol
 * 
 * For all details and documentation: https://github.com/PaulUithol/Backbone-relational.
 * Depends on (as in, compeletely useless without) Backbone: https://github.com/documentcloud/backbone.
 */
(function(undefined) {

	/**
	 * CommonJS shim
	 **/
	if (typeof window === 'undefined') {
		var _ = require('underscore');
		var Backbone = require('backbone');
		var exports = module.exports;
		exports.Backbone = Backbone;
	} else {
		var _ = this._;
		var Backbone = this.Backbone;
		var exports = this;
	}
	
	/**
	 * Semaphore mixin; can be used as both binary and counting.
	 **/
	Backbone.Semaphore = {
		_permitsAvailable: null,
		_permitsUsed: 0,
		
		acquire: function() {
			if ( this._permitsAvailable && this._permitsUsed >= this._permitsAvailable ) {
				throw new Error('Max permits acquired');
			}
			else {
				this._permitsUsed++;
			}
		},
		
		release: function() {
			if ( this._permitsUsed === 0 ) {
				throw new Error('All permits released');
			}
			else {
				this._permitsUsed--;
			}
		},
		
		isLocked: function() {
			return this._permitsUsed > 0;
		},
		
		setAvailablePermits: function( amount ) {
			if ( this._permitsUsed > amount ) {
				throw new Error('Available permits cannot be less than used permits');
			}
			this._permitsAvailable = amount;
		}
	};
	
	/**
	 * A BlockingQueue that accumulates items while blocked (via 'block'),
	 * and processes them when unblocked (via 'unblock').
	 * Process can also be called manually (via 'process').
	 */
	Backbone.BlockingQueue = function() {
		this._queue = [];
	};
	_.extend( Backbone.BlockingQueue.prototype, Backbone.Semaphore, {
		_queue: null,
		
		add: function( func ) {
			if ( this.isBlocked() ) {
				this._queue.push( func );
			}
			else {
				func();
			}
		},
		
		process: function() {
			while ( this._queue && this._queue.length ) {
				this._queue.shift()();
			}
		},
		
		block: function() {
			this.acquire();
		},
		
		unblock: function() {
			this.release();
			if ( !this.isBlocked() ) {
				this.process();
			}
		},
		
		isBlocked: function() {
			return this.isLocked();
		}
	});
	/**
	 * Global event queue. Accumulates external events ('add:<key>', 'remove:<key>' and 'update:<key>')
	 * until the top-level object is fully initialized (see 'Backbone.RelationalModel').
	 */
	Backbone.eventQueue = new Backbone.BlockingQueue();
	
	/**
	 * Backbone.Store keeps track of all created (and destruction of) Backbone.RelationalModel.
	 * Handles lookup for relations.
	 */
	Backbone.Store =  function() {
		this._collections = [];
		this._reverseRelations = [];
	};
	_.extend( Backbone.Store.prototype, Backbone.Events, {
		_collections: null,
		_reverseRelations: null,
		
		/**
		 * Add a reverse relation. Is added to the 'relations' property on model's prototype, and to
		 * existing instances of 'model' in the store.
		 * @param {object} relation; required properties:
		 *  - model, type, key and relatedModel
		 */
		addReverseRelation: function( relation ) {
			var exists = _.any( this._reverseRelations, function( rel ) {
				return _.all( relation, function( val, key ) {
					return val === rel[ key ];
				});
			});
			
			if ( !exists && relation.model && relation.type ) {
				this._reverseRelations.push( relation );
				
				if ( !relation.model.prototype.relations ) {
					relation.model.prototype.relations = [];
				}
				relation.model.prototype.relations.push( relation );
				
				this.retroFitRelation( relation );
			}
		},
		
		/**
		 * Add a 'relation' to all existing instances of 'relation.model' in the store
		 */
		retroFitRelation: function( relation ) {
			var coll = this.getCollection( relation.model );
			
			coll.each( function( model ) {
				new relation.type( model, relation );
			}, this);
		},
		
		/**
		 * Find the Store's collection for a certain type of model.
		 * @param model {Backbone.RelationalModel}
		 * @return {Backbone.Collection} A collection if found (or applicable for 'model'), or null
		 */
		getCollection: function( model ) {
			var coll =  _.detect( this._collections, function( c ) {
					// Check if model is the type itself (a ref to the constructor), or is of type c.model
					return model === c.model || model instanceof c.model;
				});
			
			if ( !coll ) {
				coll = this._createCollection( model );
			}
			
			return coll;
		},
		
		/**
		 * Find a type on the global object by name. Splits name on dots.
		 * @param {string} name
		 */
		getObjectByName: function( name ) {
			var type = _.reduce( name.split('.'), function( memo, val ) {
				return memo[ val ];
			}, exports);
			return type !== exports ? type: null;
		},
		
		_createCollection: function( type ) {
			var coll;
			
			// If 'type' is an instance, take it's constructor
			if ( type instanceof Backbone.RelationalModel ) {
				type = type.constructor;
			}
			
			// Type should inherit from Backbone.RelationalModel.
			if ( type.prototype instanceof Backbone.RelationalModel.prototype.constructor ) {
				coll = new Backbone.Collection();
				coll.model = type;
				
				this._collections.push( coll );
			}
			
			return coll;
		},
		
		find: function( type, id ) {
			var coll = this.getCollection( type );
			return coll && coll.get( id );
		},
		
		/**
		 * Add a 'model' to it's appropriate collection. Retain the original contents of 'model.collection'.
		 */
		register: function( model ) {
			var modelColl = model.collection;
			var coll = this.getCollection( model );
			coll && coll._add( model );
			model.collection = modelColl;
		},
		
		unregister: function( model ) {
			var coll = this.getCollection( model );
			coll && coll.remove( model );
		}
	});
	Backbone.store = new Backbone.Store();
	
	/**
	 * The main Relation class, from which 'HasOne' and 'HasMany' inherit.
	 * @param {Backbone.RelationalModel} instance
	 * @param {object} options.
	 *  Required properties:
	 *    - {Backbone.RelationalModel} instance
	 *    - {string} key
	 *    - {Backbone.RelationalModel.constructor} relatedModel
	 *  Optional properties:
	 *    - {bool} includeInJSON: create objects from the contents of keys if the object is not found in Backbone.store.
	 *    - {bool} createModels: serialize the attributes for related model(s)' in toJSON on create/update, or just their ids.
	 *    - {object} reverseRelation: Specify a bi-directional relation. If provided, Relation will reciprocate
	 *        the relation to the 'relatedModel'. Required and optional properties match 'options', except for:
	 *        - {string|Backbone.Relation} type: 'HasOne' or 'HasMany'
	 */
	Backbone.Relation = function( instance, options ) {
		var dit = this;
		this.instance = instance;
		
		// Make sure 'options' is sane, and fill with defaults from subclasses and this object's prototype
		options = ( typeof options === 'object' && options ) || {};
		this.reverseRelation = _.defaults( options.reverseRelation || {}, this.options.reverseRelation );
		this.reverseRelation.type = _.isString( this.reverseRelation.type ) ? Backbone[ this.reverseRelation.type ] : this.reverseRelation.type;
		this.options = _.defaults( options, this.options, Backbone.Relation.prototype.options );
		
		this.key = this.options.key;
		this.keyContents = this.instance.get( this.key );
		
		// 'exports' should be the global object where 'relatedModel' can be found on if given as a string.
		this.relatedModel = this.options.relatedModel;
		if ( _.isString( this.relatedModel ) ) {
			this.relatedModel = Backbone.store.getObjectByName( this.relatedModel );
		}
		
		if ( this.checkPreconditions() ) {
			// Add this Relation to instance._relations
			this.instance._relations.push( this );
		}
		else {
			return false;
		}
		
		// Add the reverse relation on 'relatedModel' to Backbone.store's reverseRelations
		if ( !this.options.isAutoRelation && this.reverseRelation.type && this.reverseRelation.key ) {
			Backbone.store.addReverseRelation( _.defaults( {
					isAutoRelation: true,
					model: this.relatedModel,
					relatedModel: this.instance.constructor,
					reverseRelation: this.options // current relation is the 'reverseRelation' for it's own reverseRelation
				},
				this.reverseRelation // Take further properties from this.reverseRelation (type, key, etc.)
			) );
		}
		
		// When a model in the store is destroyed, check if it is 'this.instance'.
		Backbone.store.getCollection( this.instance ).bind( 'remove', function( model ) {
				if ( model === dit.instance ) {
					dit.destroy();
				}
			});
		
		// When 'relatedModel' are created or destroyed, check if it affects this relation.
		Backbone.store.getCollection( this.relatedModel )
			.bind( 'add', function( model, coll, options ) {
					// Allow 'model' to set up it's relations, before calling 'tryAddRelated'
					// (which can result in a call to 'addRelated' on a relation of 'model')
					model.queue( function() {
						dit.tryAddRelated( model, options );
					});
				})
			.bind( 'remove', function( model, coll, options ) {
					dit.removeRelated( model, options );
				});
		
		this.initialize();
	};
	// Fix inheritance :\
	Backbone.Relation.extend = Backbone.Model.extend;
	// Set up all inheritable **Backbone.Relation** properties and methods.
	_.extend( Backbone.Relation.prototype, Backbone.Events, Backbone.Semaphore, {
		options: {
			createModels: true,
			includeInJSON: true,
			isAutoRelation: false
		},
		
		instance: null,
		key: null,
		keyContents: null,
		relatedModel: null,
		reverseRelation: null,
		related: null,
		
		/**
		 * Check several pre-conditions.
		 * @return {bool} True if pre-conditions are satisfied, false if they're not.
		 */
		checkPreconditions: function() {
			var i = this.instance, k = this.key, rm = this.relatedModel;
			if ( !i || !k || !rm ) {
				console && console.warn( 'Relation=%o; no instance, key or relatedModel (%o, %o, %o)', this, i, k, rm );
				return false;
			}
			// Check if 'instance' is a Backbone.RelationalModel
			if ( !( i instanceof Backbone.RelationalModel ) ) {
				console && console.warn( 'Relation=%o; instance=%o is not a Backbone.RelationalModel', this, i );
				return false;
			}
			// Check if the type in 'relatedModel' inherits from Backbone.RelationalModel
			if ( !( rm.prototype instanceof Backbone.RelationalModel.prototype.constructor ) ) {
				console && console.warn( 'Relation=%o; relatedModel does not inherit from Backbone.RelationalModel (%o)', this, rm );
				return false;
			}
			// Check if this is not a HasMany, and the reverse relation is HasMany as well
			if ( this instanceof Backbone.HasMany && this.reverseRelation.type === Backbone.HasMany.prototype.constructor ) {
				console && console.warn( 'Relation=%o; relation is a HasMany, and the reverseRelation is HasMany as well.', this );
				return false;
			}
			// Check if we're not attempting to create a duplicate relationship
			if ( i._relations.length ) {
				var exists = _.any( i._relations, function( rel ) {
					var hasReverseRelation = this.reverseRelation.key && rel.reverseRelation.key;
					return rel.relatedModel === rm && rel.key === k
						&& ( !hasReverseRelation || this.reverseRelation.key === rel.reverseRelation.key );
				}, this );
				
				if ( exists ) {
					console && console.warn( 'Relation=%o between instance=%o.%s and relatedModel=%o.%s already exists',
						this, i, k, rm, this.reverseRelation.key );
					return false;
				}
			}
			return true;
		},
		
		setRelated: function( related, options ) {
			this.related = related;
			var value = {};
			value[ this.key ] = related;
			this.instance.acquire();
			this.instance.set( value, _.defaults( options || {}, { silent: true } ) );
			this.instance.release();
		},
		
		createModel: function( item ) {
			if ( this.options.createModels && typeof( item ) === 'object' ) {
				return new this.relatedModel( item );
			}
		},
		
		/**
		 * Determine if a relation (on a different RelationalModel) is the reverse
		 * relation of the current one.
		 */
		_isReverseRelation: function( relation ) {
			if ( relation.instance instanceof this.relatedModel && this.reverseRelation.key === relation.key
					&&	this.key === relation.reverseRelation.key ) {
				return true;
			}
			return false;
		},
		
		/**
		 * Get the reverse relations (pointing back to 'this.key' on 'this.instance') for the currently related model(s).
		 * @param model {Backbone.RelationalModel} Optional; get the reverse relations for a specific model.
		 *		If not specified, 'this.related' is used.
		 */
		getReverseRelations: function( model ) {
			var reverseRelations = [];
			// Iterate over 'model', 'this.related.models' (if this.related is a Backbone.Collection), or wrap 'this.related' in an array.
			var models = !_.isUndefined( model ) ? [ model ] : this.related && ( this.related.models || [ this.related ] );
			_.each( models , function( related ) {
				_.each( related.getRelations(), function( relation ) {
					if ( this._isReverseRelation( relation ) ) {
						reverseRelations.push( relation );
					}
				}, this );
			}, this );
			
			return reverseRelations;
		},
		
		// Cleanup. Get reverse relation, call removeRelated on each.
		destroy: function() {
			_.each( this.getReverseRelations(), function( relation ) {
					relation.removeRelated( this.instance );
				}, this );
		}
	});
	
	Backbone.HasOne = Backbone.Relation.extend({
		options: {
			reverseRelation: { type: 'HasMany' }
		},
		
		initialize: function() {
			_.bindAll( this, 'onChange' );
			this.instance.bind( 'relationChange:' + this.key, this.onChange );
			
			var model = this.findRelated();
			this.setRelated( model );
			
			// Notify new 'related' object of the new relation.
			var dit = this;
			_.each( dit.getReverseRelations(), function( relation ) {
					relation.addRelated( dit.instance );
				} );
		},
		
		findRelated: function() {
			var item = this.keyContents;
			var model = null;
			
			if ( item instanceof this.relatedModel ) {
				model = item;
			}
			else if ( item && ( _.isString( item ) || typeof( item ) === 'object' ) ) {
				// Try to find an instance of the appropriate 'relatedModel' in the store, or create it
				var id = _.isString( item ) ? item : item[ this.relatedModel.prototype.idAttribute ];
				model = Backbone.store.find( this.relatedModel, id ) || this.createModel( item );
			}
			
			return model;
		},
		
		/**
		 * If the key is changed, notify old & new reverse relations and initialize the new relation
		 */
		onChange: function( model, attr, options ) {
			// Don't accept recursive calls to onChange (like onChange->findRelated->createModel->initializeRelations->addRelated->onChange)
			if ( this.isLocked() ) {
				return;
			}
			this.acquire();
			
			// 'options._related' is set by 'addRelated'/'removeRelated'. If it is set, the change
			// is the result of a call from a relation. If it's not, the change is the result of 
			// a 'set' call on this.instance.
			var changed = _.isUndefined( options._related );
			var oldRelated = changed ? this.related : options._related;
			
			if ( changed ) {	
				this.keyContents = attr;
				
				// Set new 'related'
				if ( attr instanceof this.relatedModel ) {
					this.related = attr;
				}
				else if ( attr ) {
					var related = this.findRelated();
					this.setRelated( related );
				}
				else {
					this.setRelated( null );
				}
			}
			
			// Notify old 'related' object of the terminated relation
			if ( oldRelated && this.related !== oldRelated ) {
				_.each( this.getReverseRelations( oldRelated ), function( relation ) {
						relation.removeRelated( this.instance, options );
					}, this );
			}
			
			// Notify new 'related' object of the new relation. Note we do re-apply even if this.related is oldRelated;
			// that can be necessary for bi-directional relations if 'this.instance' was created after 'this.related'.
			// In that case, 'this.instance' will already know 'this.related', but the reverse might not exist yet.
			_.each( this.getReverseRelations(), function( relation ) {
					relation.addRelated( this.instance, options );
				}, this);
			
			// Fire the 'update:<key>' event if 'related' was updated
			if ( !options.silentChange && this.related !== oldRelated ) {
				var dit = this;
				Backbone.eventQueue.add( function() {
					dit.instance.trigger( 'update:' + dit.key, dit.instance, dit.related, options );
				});
			}
			this.release();
		},
		
		/**
		 * If a new 'this.relatedModel' appears in Backbone.store, try to match it to the last set 'keyContents'
		 */
		tryAddRelated: function( model, options ) {
			if ( this.related ) {
				return;
			}
			
			var item = this.keyContents;
			if ( item && ( _.isString( item ) || typeof( item ) === 'object' ) ) {
				var id = _.isString( item ) ? item : item[ this.relatedModel.prototype.idAttribute ];
				if ( model.id === id ) {
					this.addRelated( model, options );
				}
			}
		},
		
		addRelated: function( model, options ) {
			if ( model !== this.related ) {
				var oldRelated = this.related || null;
				this.setRelated( model );
				this.onChange( this.instance, model, { _related: oldRelated } );
			}
		},
		
		removeRelated: function( model, options ) {
			if ( !this.related ) {
				return;
			}
			
			if ( model === this.related ) {
				var oldRelated = this.related || null;
				this.setRelated( null );
				this.onChange( this.instance, model, { _related: oldRelated } );
			}
		}
	});
	
	Backbone.HasMany = Backbone.Relation.extend({
		options: {
			reverseRelation: { type: 'HasOne' }
		},
		
		initialize: function() {
			_.bindAll( this, 'onChange', 'handleAddition', 'handleRemoval' );
			this.instance.bind( 'relationChange:' + this.key, this.onChange );
			
			this.setRelated( this.getNewCollection() );
			this.findRelated();
		},
		
		getNewCollection: function() {
			if ( this.related ) {
				this.related.unbind( 'add', this.handleAddition ).unbind('remove', this.handleRemoval );
			}
			
			var related = new Backbone.Collection();
			related.model = this.relatedModel;
			related.bind( 'add', this.handleAddition ).bind('remove', this.handleRemoval );
			return related;
		},
		
		findRelated: function() {
			if ( this.keyContents && _.isArray( this.keyContents ) ) {
				// Try to find instances of the appropriate 'relatedModel' in the store
				_.each( this.keyContents, function( item ) {
					var id = _.isString( item ) ? item : item[ this.relatedModel.prototype.idAttribute ];
					var model = Backbone.store.find( this.relatedModel, id ) || this.createModel( item );
					
					if ( model && !this.related.getByCid( model ) && !this.related.get( model ) ) {
						this.related._add( model );
					}
				}, this);
			}
		},
		
		/**
		 * If the key is changed, notify old & new reverse relations and initialize the new relation
		 */
		onChange: function( model, attr, options ) {
			this.keyContents = attr;
			
			// Notify old 'related' object of the terminated relation
			_.each( this.getReverseRelations(), function( relation ) {
					relation.removeRelated( this.instance, options );
				}, this );
			
			// Set new 'related'
			if ( attr instanceof Backbone.Collection ) {
				this.related.unbind( 'add', this.handleAddition ).unbind('remove', this.handleRemoval );
				this.related = attr;
				this.related.bind( 'add', this.handleAddition ).bind( 'remove', this.handleRemoval );
			}
			else {
				this.setRelated( this.getNewCollection() );
				this.findRelated();
			}
			
			// Notify new 'related' object of the new relation
			_.each( this.getReverseRelations(), function( relation ) {
					relation.addRelated( this.instance, options );
				}, this );
			
			var dit = this;
			Backbone.eventQueue.add( function() {
				!options.silentChange && dit.instance.trigger( 'update:' + dit.key, dit.instance, dit.related, options );
			});
		},
		
		tryAddRelated: function( model, options ) {
			if ( !this.related.getByCid( model ) && !this.related.get( model ) ) {
				// Check if this new model was specified in 'this.keyContents'
				var item = _.any( this.keyContents, function( item ) {
					var id = _.isString( item ) ? item : item[ this.relatedModel.prototype.idAttribute ];
					return id && id === model.id;
				}, this );
				
				if ( item ) {
					this.related._add( model, options );
				}
			}
		},
		
		/**
		 * When a model is added to a 'HasMany', trigger 'add' on 'this.instance' and notify reverse relations.
		 * (should be 'HasOne', must set 'this.instance' as their related).
		 */
		handleAddition: function( model, coll, options ) {
			options = options || {};
			var dit = this;
			
			_.each( this.getReverseRelations( model ), function( relation ) {
					relation.addRelated( this.instance, options );
				}, this );
			
			// Only trigger 'add' once the newly added model is initialized (so, has it's relations set up)
			Backbone.eventQueue.add( function() {
				!options.silentChange && dit.instance.trigger( 'add:' + dit.key, model, dit.related, options );
			});
		},
		
		/**
		 * When a model is removed from a 'HasMany', trigger 'remove' on 'this.instance' and notify reverse relations.
		 * (should be 'HasOne', which should be nullified)
		 */
		handleRemoval: function( model, coll, options ) {
			options = options || {};
			
			_.each( this.getReverseRelations( model ), function( relation ) {
					relation.removeRelated( this.instance, options );
				}, this );
			
			var dit = this;
			Backbone.eventQueue.add( function() {
				!options.silentChange && dit.instance.trigger( 'remove:' + dit.key, model, dit.related, options );
			});
		},
		
		addRelated: function( model, options ) {
			var dit = this;
			model.queue( function() { // Queued to avoid errors for adding 'model' to the 'this.related' set twice
				if ( !dit.related.getByCid( model ) && !dit.related.get( model.id ) ) {
					dit.related._add( model, options );
				}
			});
		},
		
		removeRelated: function( model, options ) {
			if ( this.related.getByCid( model ) || this.related.get( model.id ) ) {
				this.related.remove( model, options );
			}
		}
	});
	
	/**
	 * New events:
	 *  - 'add:<key>' (model, coll)
	 *  - 'remove:<key>' (model, coll)
	 */
	Backbone.RelationalModel = Backbone.Model.extend({
		relations: null, // Relation descriptions on the prototype
		_relations: null, // Relation instances
		_isInitialized: false,
		_deferProcessing: false,
		_queue: null,
		
		constructor: function( attributes, options ) {
			// Nasty hack, for cases like 'model.get( <HasMany key> ).add( item )'.
			// Defer 'processQueue', so that when 'Relation.createModels' is used we:
			// a) Survive 'Backbone.Collection._add'; this takes care we won't error on "can't add model to a set twice"
			//    (by creating a model from properties, having the model add itself to the collection via one of
			//    it's relations, then trying to add it to the collection).
			// b) Trigger 'HasMany' collection events only after the model is really fully set up.
			// Example that triggers both a and b: "p.get('jobs').add( { company: c, person: p } )".
			var dit = this;
			if ( options && options.collection ) {
				this._deferProcessing = true;
				
				var processQueue = function( model, coll ) {
					if ( model === dit ) {
						dit._deferProcessing = false;
						dit.processQueue();
						options.collection.unbind( 'add', processQueue );
					}
				};
				options.collection.bind( 'add', processQueue );
				
				// So we do process the queue eventually, regardless of whether this model really gets added to 'options.collection'.
				_.defer( function() {
					processQueue( dit );
				});
			}
			
			this._queue = new Backbone.BlockingQueue();
			this._queue.block();
			Backbone.eventQueue.block();
			
			Backbone.Model.prototype.constructor.apply( this, arguments );
			
			// Try to run the global queue holding external events
			Backbone.eventQueue.unblock();
		},
		
		/**
		 * Initialize Relations present in this.relations; determine the type (HasOne/HasMany), then create a new instance.
		 * The regular constructor (which fills this.attributes, initialize, etc) hasn't run yet at this time!
		 */
		initializeRelations: function() {
			this.acquire(); // Lock; setting up relations often also involve calls to 'set'
			this._relations = [];
			
			Backbone.store.register( this );
			
			_.each( this.relations, function( rel ) {
				var type = rel.type && ( _.isString( rel.type ) ? Backbone[ rel.type ] : rel.type );
				if ( type.prototype instanceof Backbone.Relation.prototype.constructor ) {
					new type( this, rel ); // Also pushes the new Relation into _relations
				}
			}, this );
			
			this._isInitialized = true;
			this.release();
			this.processQueue();
		},
		
		/**
		 * Either add to the queue (if we're not initialized yet), or execute right away.
		 */
		queue: function( func ) {
			this._queue.add( func );
		},
		
		/**
		 * Process _queue
		 */
		processQueue: function() {
			if ( this._isInitialized && !this._deferProcessing && this._queue.isBlocked() ) {
				this._queue.unblock();
			}
		},
		
		getRelations: function() {
			return this._relations;
		},
		
		set: function( attributes, options ) {
			Backbone.eventQueue.block();
			
			var result = Backbone.Model.prototype.set.apply( this, arguments );
			
			// When new values are set, notify this model's relations (also if options.silent is set).
			// Relation.setRelated locks this model before calling 'set' on it to prevent loops.
			if( this._isInitialized && !this.isLocked() ) {
				// Rename options.silent, so add/remove events propagate properly in HasMany
				// relations from 'addRelated'->'handleAddition'
				if ( options && options.silent ) {
					options.silentChange = options.silent;
					delete options.silent;
				}
				
				_.each( this._relations, function( rel ) {
					var val = this.attributes[ rel.key ];
					if ( rel.related !== val ) {
						this.trigger('relationChange:' + rel.key, this, val, options || {} );
					}
				}, this );
			}
			
			// 'set' is called quite late in 'Backbone.Model.prototype.constructor', but before 'initialize'.
			// Ideal place to set up relations :)
			if ( !this._isInitialized && !this.isLocked() && this.relations ) {
				this.initializeRelations();
			}
			
			// Try to run the global queue holding external events
			Backbone.eventQueue.unblock();
			
			return result;
		},
		
		destroy: function( options ) {
			Backbone.store.unregister( this );
			return Backbone.Model.prototype.destroy.call( this, options );
		},
		
		/**
		 * Convert relations to JSON, omits them when required
		 */
		toJSON: function() {
			// If this Model has already been fully serialized in this branch once, return to avoid loops
			if ( this.isLocked() ) {
				return this.id;
			}
			
			var json = Backbone.Model.prototype.toJSON.call( this );
			
			_.each( this._relations, function( rel ) {
					var value = json[ rel.key ];
					
					if ( rel.options.includeInJSON && value && _.isFunction( value.toJSON ) ) {
						this.acquire();
						json[ rel.key ] = value.toJSON();
						this.release();
					}
					else if ( value instanceof Backbone.Collection ) {
						json[ rel.key ] = value.pluck( value.model.prototype.idAttribute );
					}
					else if ( value instanceof Backbone.Model ) {
						json[ rel.key ] = value.id;
					}
				}, this );
			
			return json;
		}
	});
	_.extend( Backbone.RelationalModel.prototype, Backbone.Semaphore );
	
	// Override Backbone.Collection._add, so objects fetched from the server multiple times will
	// update the existing Model.
	var _add = Backbone.Collection.prototype._add;
	Backbone.Collection.prototype._add = function( model, options ) {
		if ( !( model instanceof Backbone.Model ) ) {
			// Try to find 'model' in Backbone.store. If it already exists, set the new properties on it.
			var found = Backbone.store.find( this.model, model[ this.model.prototype.idAttribute ] );
			if ( found ) {
				model = found.set( model, options );
			}
		}
		return _add.call( this, model, options );
	};
})();
